/* ************************************************* ;
* Macro LoD_Est                                        
* Calculates the LoD based on hit-rate data inputs 
* Includes Percent Detection Table
*                 
* Developer:  Jesse A. Canchola
*                                 
* Macro Inputs (with examples):     
*                               
* datain      = LoD      ; * enter data set name ;
* study       = YourStudy; * enter study name or investigator name ;
* target      = CMV      ; * enter target/virus ;
* subset      = target="CMV" ; * enter "where" statement language ;
* Units       = IU/mL    ; * enter units of measurement: IU/mL, copies/mL, cp/mL, etc. ;
* distn       = Probit   ; * enter EXTREME, EXTREMEVALUE, GOMPERTZ, GOMPERZ, LOGISTIC, PROBIT ;
* alpha       = 0.05     ; * significance level ;
* sided       = 2        ; * 1 or 2 sided interval ;
* TableNo     = 1        ; * Table Number ;
* FigureNo    = 2        ; * Figure Number ;
* ODStype     = RTF      ; * enter ODS output type: RTF, PDF, HTML, etc. ;
* RTFout      = B:\SAS\Out ; * output &ODStype file ;

* GraphMin    = -1.0 ; * enter on log10 scale ;
* GraphMax    =  3.0 ; * enter on log10 scale ;
* GraphByInc  =  0.5 ; * enter scale increment ;

* GraphTitle1 = Figure &FigureNo.. Limit of Detection Estimation for &target. Target ; * main title at top of graph ;
* GraphTitle2 = &distn Curve Fit ; * smaller title below main title at top of graph ;
* YaxisTitle  = Hit Rate ; * Y-axis title: Hit Rate, Detection Rate, Probability, etc. 
* XaxisTitle  = Titer (log10 &Units) ; * X-axis title: Log10 Titer, Log Titer, Titer 
*
************************************************** */ ;
%MACRO LoD_Est (
datain      = ,
study       = ,
target      = ,
subset      = ,
Units       = ,
distn       = ,
alpha       = ,
sided       = ,
TableNo     = ,
FigureNo    = ,
ODStype     = ,
RTFout      = ,
GraphMin    = ,
GraphMax    = ,
GraphByInc  = ,
GraphTitle1 = ,
GraphTitle2 = ,
YaxisTitle  = ,
XaxisTitle  = 
) ;

proc format ;
 picture mltdecaf (multilabel)
         0-9     = '9.9'
         10-99   = '99' 
		 100-999 = '999' 
	   1000-9999 = '9999' ;
 picture mltdecbf (multilabel)
         0-9.9     = '9.99'
         10-99.9   = '99.9'
		 100-999.9 = '999' 
       1000-9999.9 = '9999' ;
run ;

proc sort data = &DataIn ; by Target ; run ;

data DataIn ;
Set &DataIn ;
where &subset ;
run ;


* Use the MODSTYLE macro supplied by SAS to change the default colors and markers ;
 %modstyle(name    = markers , 
           parent  = listing ,
           type    = CLM     ,
           markers = circle triangle square) ; 


ods graphics on / outputfmt=EMF ;
ods _ALL_ close ;
ods &ODStype file = "&RTFout" gtitle style = markers ; * style=analysis ;

proc sort data=DataIn ; by Target ; run ;

*ods listing sge=on style=Analysis ;
ods noproctitle ;
ods output ProbitAnalysis=PredProbs GoodnessOfFit=GOF ;

%IF %upcase(&distn) = PROBIT %THEN %DO ;
 title1 "LoD Estimation with Probit Analysis" ;
 title2 "  " ;
 proc probit data = DataIn lackfit ; 
   model Hits / TotalTests = log10_titer 
              / inversecl d = Normal ;
   predpplot var = log10_titer vref = 0.95 ;
   by Target ; 
 run ;
%END ;
%ELSE %DO ;
 title1 "LoD Estimation with &distn Analysis" ;
 title2 "  " ;
 proc probit data = DataIn lackfit ; 
   model Hits / TotalTests = log10_titer 
              / inversecl d = &distn ;
   predpplot var = log10_titer vref = 0.95 ;
   by Target ; 
 run ;
%END ;

* combine/concatenate original LoD with PredProbs to overlay in SGPLOT ;
data ComboLoD_Target1 ;
set DataIn PredProbs ( rename=(TARGET=TARGET1) ) ;
run ;

* extracting LoD at 95% and associated 95% CI ;
data Estimates ;
set PredProbs ( rename = (Variable=LogLoDEst LowerCL=LogLCL UpperCL=LogUCL) ) ;

* Get anti-log for actual value in normal units ;
LoD_Est = 10**LogLoDEst ;
LCL95   = 10**LogLCL    ;
UCL95   = 10**LogUCL    ;

* convert to character with 4.2 format since numeric is out to more decimals and will not work directly ;
ProbabilityChar = put(Probability,4.2) ; 

* Keep only one line at 95% hit rate ;
if ProbabilityChar = "0.95" ; 

* output values as macro vars for use in Proc SGPLOT ;
call symput("LogLoDEst",LogLoDEst) ;
call symput("LogLCL",LogLCL) ;
call symput("LogUCL",LogUCL) ;

call symput("LoD_Est" , compress(round(LoD_Est,0.1))) ;
call symput("LCL95"   , compress(round(LCL95  ,0.1))) ;
call symput("UCL95"   , compress(round(UCL95  ,0.1))) ;
run;
proc print data = Estimates ; run ;

* Extract Goodness-of-Fit statistics ;
* Name = "Pearson Chi-Square" 
         "L.R.    Chi-Square"
Value
DF
ProbChiSq * ;
data GOFstats ; set GOF ; run ;
Proc IML ;
use GOFstats ;
read ALL VAR{Value DF ProbChiSq} into X ;
close GOFstats ;

PearsonStat = round(X[1,1],0.1) ;
PearsonDF   = round(X[1,2],0.1) ;
PearsonPval = round(X[1,3],0.01) ;

LRStat = round(X[2,1],0.1) ;
LRDF   = round(X[2,2],0.1) ;
LRPval = round(X[2,3],0.01) ;

print PearsonStat LRStat ;

call symputx("PearsonStat" ,PearsonStat) ;
call symputx("PearsonDF"   ,PearsonDF  ) ;
call symputx("PearsonPval" ,PearsonPval) ;

call symputx("LRStat" ,LRStat) ;
call symputx("LRDF"   ,LRDF  ) ;
call symputx("LRPval" ,LRPval) ;
run ;
proc print data=GOFstats ; run ;

ods graphics on / width = 550px height = 550px ;  
proc sgplot data = ComboLoD_Target1 ;
  scatter x=Log10_Titer y=Obs_HitRate / LEGENDLABEL="Observed Hit Rate"  MARKERATTRS=(SIZE=7px) name="Assay1" ;    * group=Target grouporder=ascending  COLOR=blue SYMBOL=triangle) ;

  series x = variable y = Probability / MARKERATTRS=(SIZE=7px) LINEATTRS=(THICKNESS=0.2 COLOR=blue   PATTERN=1) name="HitRate" ;  */ CurveLabel = "Probit"  ;
  series x = lowercl  y = Probability / MARKERATTRS=(SIZE=7px) LINEATTRS=(THICKNESS=0.2 COLOR=red    PATTERN=2) name="LowerCL" ;  */ CurveLabel = "95% LCL" ;
  series x = uppercl  y = Probability / MARKERATTRS=(SIZE=7px) LINEATTRS=(THICKNESS=0.2 COLOR=green  PATTERN=3) name="UPPERCL" ;  */ CurveLabel = "95% UCL" ;
  
  * Reference line at 0.95 probability ;
  REFLINE 0.95       / AXIS=Y TRANSPARENCY = 0.5 LINEATTRS = (THICKNESS=0.2 COLOR=blue PATTERN=1) LABEL = "95% &YaxisTitle" ; * Add reference line to 95% ;

    * Reference line at LoD (log) ;
  REFLINE &LogLoDEst / AXIS=X TRANSPARENCY = 0.5 LINEATTRS = (THICKNESS=0.2 COLOR=black PATTERN=4) LABEL = "" ; 

  * Reference line at lower 95% CI for LoD (log) ;
  REFLINE &LogLCL    / AXIS=X TRANSPARENCY = 0.5 LINEATTRS = (THICKNESS=0.2 COLOR=orange PATTERN=5) LABEL = "" ; 

  * Reference line at upper 95% CI for LoD (log) ;
  REFLINE &LogUCL    / AXIS=X TRANSPARENCY = 0.5 LINEATTRS = (THICKNESS=0.2 COLOR=purple PATTERN=6) LABEL = "" ; 

  YAXIS LABEL = "&YaxisTitle"  VALUES = (0  TO 1   BY 0.1) ;
  XAXIS LABEL = "&XaxisTitle"  VALUES = (&GraphMin TO &GraphMax BY &GraphByInc) ;

  KEYLEGEND "Assay1"  / LOCATION=outside POSITION=bottom title=" LoD Est= &LoD_Est &Units   95% CI= ( &LCL95, &UCL95 ) " ;

  INSET "Goodness-of-Fit Statistics: "
        "                            "
        "Pearson Chisq= &PearsonStat  P-value= &PearsonPval" 
        "LR Chisq          = &LRStat  P-value= &LRPval     " / BORDER TEXTATTRS=(Color=black Family=Arial Size=6px Style=Italic Weight=Bold) ; *POSITION = TOPLEFT ; 


  TITLE1 "&GraphTitle1" ;
  TITLE2 "&GraphTitle2" ;
run ;

* ****************************************** * ;
* Hit Rate Table * ;
* ****************************************** * ;
data DataIn1 ;
set DataIn ;

* 2-sided 95% Clopper-Pearson CI for hit rates ;
if &sided = 1 then do ; 
  zalpha = probit( 1 - ( &alpha ) ) ; 
  end ; 
  else if &sided = 2 then do ; 
  zalpha = probit( 1 - ( &alpha / 2 ) ) ; 
end ; 

n = TotalTests ;
p = Hits / TotalTests ;

** Exact Clopper Pearson ; 
 x = round( n * p, 0.1 ) ; 
 
 * Calculate the lower limit. ; 
 v1 = 2 * ( n - x + 1 ) ; 
 v2 = 2 * x ; 
 
 if &sided = 1 then do ; 
 a = 1 - ( &alpha ) ; 
 end ; 
 else if &sided = 2 then do ; 
 a = 1 - ( &alpha / 2 ) ; 
 end ; 
 
 coef = ( n - x + 1 ) / x ; 
 fscore = finv( a, v1, v2 ) ; 
 exact_lcl = 1 / ( 1 + coef * fscore ) ; 
 
 * Calculate the upper limit. ; 
 v11 = 2 * ( x + 1 ) ; 
 v22 = 2 * ( n - x ) ; 
 fscore2 = finv( a, v11, v22 ) ; 
 coef2 = ( x + 1 )/(n-x) ; 
 numer = coef2 * fscore2 ; 
 exact_ucl = numer / ( 1 + numer ) ; 
 if exact_ucl = . & p = 1 then exact_ucl = 1.0  ; * check this ;
 if exact_lcl = .         then exact_lcl = 9999 ;

 * converting ci to percent AND HANDLING MISSING DATA IF ANY ;
 if exact_lcl ne 9999 then exact_lclp = exact_lcl * 100 ;
 if exact_ucl ne 9999 then exact_uclp = exact_ucl * 100 ;
* ****************** end Clopper-Pearson CI calculation ******************* ;

drop N p x V1 V2 A Coef Fscore V11 V22 Fscore2 Coef2 Numer ;
run ;


* Proc report ;
options  orientation = landscape nodate nonumber ;

ods listing close ;
ods escapechar='~' ;
proc report data = DataIn1 split='*' missing headline headskip spanrows 
	style(report)={just=center outputwidth=10 in}
    style(lines) =header{font_size=9pt font_face="Arial" font_weight=medium background=transparent just=left}
    style(header)=header{font_size=9pt font_face="Arial" font_weight=bold   background=transparent just=center}
    style(column)=header{font_size=9pt font_face="Arial" font_weight=medium background=transparent just=center};            

columns Target Titer Log10_Titer Hits TotalTests Obs_HitPcnt Exact_LCLP Exact_UCLP ;

define Target / display "Target"  group order=freq 
  style(header)= {just=center cellwidth=.6 in}
  style(column)= {just=center cellwidth=.6 in vjust=center} ;

define Titer / display "Titer (&units)" f=mltdecaf. 
  style(header)={just=center cellwidth=.55 in}
  style(column)={protectspecialchars=off just=center /*pretext="\qj\tqdec\tx600 "*/ cellwidth=.55 in} ;

define Log10_Titer / display "Titer*(log10 &units)" f=7.4
  style(header)={just=center cellwidth=.55 in}
  style(column)={protectspecialchars=off just=center /*pretext="\qj\tqdec\tx600 "*/ cellwidth=.55 in} ;

define Hits / display "No. Detected" f=6.0
  style(header)={just=center cellwidth=.55 in}
  style(column)={protectspecialchars=off just=center /*pretext="\qj\tqdec\tx600 "*/ cellwidth=.55 in} ;

define TotalTests  / display "Total Tests" f=5.0 
  style(header)={just=center cellwidth=.5 in}
  style(column)={protectspecialchars=off just=center /*pretext="\qj\tqdec\tx500 "*/ cellwidth=.5 in} ;

define Obs_HitPcnt  / display "Percent*Detection" f=mltdecbf.
  style(header)={just=center cellwidth=.5 in}
  style(column)={protectspecialchars=off just=center /*pretext="\qj\tqdec\tx500 "*/ cellwidth=.5 in} ;

define Exact_LCLP / display "95% Exact LL" f=mltdecbf.
  style(header)={just=center cellwidth=.5 in}
  style(column)={protectspecialchars=off just=center /*pretext="\qj\tqdec\tx500 "*/ cellwidth=.5 in} ;

define Exact_UCLP  / display "95% Exact UL" f=mltdecbf.
  style(header)={just=center cellwidth=.5 in}
  style(column)={protectspecialchars=off just=center /*pretext="\qj\tqdec\tx500 "*/ cellwidth=.5 in} ;

title1 bold j=center height=12pt f='Times' "Table &TableNo.. Percent Detection" ;

run ;

ods &ODStype close ;
%MEND LoD_Est ;